-- Tunnel Frog --
-- Main script file --

require "train"
require "shoot"
require "ui"
require "audio"

sensitivity = 3
SCREENSCALE = 4
Theme= "default"
themes = {"default", "grass", "road"}
t = 1
Vibrate = false
lines = 3
Paused = false
State = 0
lastState = 0
characters = " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.!?"

player = {x = 45, y = 128, i = 1, img = 0, char = 0, shoot = false, done = false, protected = false, hunger = 0, hungerTime = 1}
input = {x = 0, y = 0, mx = 0, move = true, moved = false}
Level = 1
Difficulty = 0.4 -- must be changeable
gStage = 1
winStage = 4
Time = 0
Timer = 0
Timeout = 10
lastHungerTime = 0
hungerMultiplier = 2 -- changes amount of increment/decremeny of hunger meter
hBarPad = 6
Score = 0
local blinking = false -- for blinking sheild
-- Debug variables --
dbug = false
db = {a = 0, b = 0, c = 0}

-- Love functions --
function love.load()
	love.graphics.setDefaultFilter("nearest")
	State = "menu"
	reset()
	audioInit()
end

function love.update(dt)
	SCREENSCALE = love.graphics.getWidth()/bg:getWidth()
	player.y = bg:getHeight() - 48
	
	if not Paused then
		if not player.done then playerUpdate(dt) end
		trainUpdate(dt)
		shootUpdate()
	end
	
	uiUpdate(dt)
end

function love.draw()
	love.graphics.scale(SCREENSCALE)
	if State ~= "menu" and State ~= "info" then
		love.graphics.draw(bg, 0, 0)
		shootDraw()
		draw(player.img, player.x, player.y)
		if player.protected and not player.done and not blinking then draw(shield, player.x, player.y) end
		trainDraw()
		bugDraw()
	end
	uiDraw()

	debug()
end
--####--

-- Events --
function love.touchpressed(id, x, y, dx, dy, p)
	player.shoot = true
	input.x, input.y = x, y
end

function love.touchmoved(id, x, y, dx, dy, p)
	if not Paused and not input.moved then
		if dx > sensitivity then
			input.mx = 1
			input.move = true
			player.shoot = false
		elseif dx < (-1 * sensitivity) then
			input.mx = -1
			input.move = true
			player.shoot = false
		else
			input.mx = 0
			input.move = false
		end
	end
	if State == "info" then
		Scroll = Scroll + dy/SCREENSCALE
		Scroll = clamp(Scroll, -96, 0)
	end
end

function love.touchreleased(id, x, y, dx, dy, p)
	input.moved = false
	player.shoot = false
	uiInput(x, y)
end

function love.keypressed(key, scancode, isrepeat)
	if key == "escape" then
		if State == "play" or State == "pause" then pause()
		elseif State == "info" or State == "win" or State == "gameover" then
			State = "menu"
			love.graphics.setFont(font)
			player.img = alive
		elseif State == "settings" then
			State = lastState
			lastState = ""
		elseif State == "menu" then
			love.system.vibrate(1)
			love.event.quit()
		end
	end
end
--####--

-- Functions --
function playerUpdate(dt)
	local x, y = bg:getDimensions()
	local px, py = player.img:getDimensions()
	
	if input.move and not input.moved then
		player.i = player.i + input.mx
		input.move = false
		input.moved = true
		input.mx = 0
		
		player.i = clamp(player.i, 0, (lines-1))
		player.x = x/(lines*2) * ((player.i * 2) + 1)
	end
	
	Time = Time + dt
	
	-- Hunger bar update --
	if math.floor(Time)%player.hungerTime == 0 and math.floor(Time) ~= lastHungerTime then
		player.hunger = clamp(player.hunger + Level*hungerMultiplier/2, hBarPad, bx-hBarPad)
		lastHungerTime = math.floor(Time)
	end
	
	if player.hunger == (bx-hBarPad) then gameOver() end
	if player.hunger <= hBarPad then gameWin() end
	
	-- Power-up reset code --
	if player.protected then
		if Time - Timer > Timeout then
			player.protected = false
			blinking = false
		elseif (Time - Timer) > (Timeout - 3) then
			Powerlow:play()
			if Time%0.2 < 0.1 then blinking = true
			else blinking = false end
		else
			blinking = false
		end
	end
end

function frogChange()
	alive = love.graphics.newImage("assets/frogs/1-"..player.char..".png")
	ded = love.graphics.newImage("assets/frogs/3-"..player.char..".png")
	player.img = alive
end

function draw(img, x, y, sx, sy)
	local dx, dy = img:getDimensions()
	local px = (x - (dx/2))
	local py = (y - (dy/2))
	sx = sx or 1
	sy = sy or 1
	love.graphics.draw(img, px, py, 0, sx, sy)
end

function debug()
	if dbug then
		love.graphics.print(db.a, 0, 0)
		love.graphics.print(db.b, 0, 17)
		love.graphics.print(db.c, 0, 34)
	end
end

function clamp(x, m, M)
	if x > M then
		x = M
	elseif x < m then
		x = m
	end
	return x
end

function biasedRandom(weights)
    local totals = {}
    local runningTotal = 0
    
    for i, w in ipairs(weights) do
        runningTotal = runningTotal + w
        table.insert(totals, runningTotal)
    end
    
    local rnd = math.random() * runningTotal
    
    for i, total in ipairs(totals) do
        if rnd < total then
            return i-1
        end
    end
end

function newTheme()
	t = t + 1
	if t > 3 then t = 1 end
	return themes[t]
end

function reset()
	Paused = true
	Score = 0
	Level = 1
	gStage = 1
	player.i = 1
	player.shoot = false
	player.done = false
	player.protected = false
	Time = 0
	Timer = 0
	input.mx = 0
	input.move = true
	shield = love.graphics.newImage("assets/"..Theme.."/Shield.png")
	bg = love.graphics.newImage("assets/"..Theme.."/0.png")
	bx, by = bg:getDimensions()
	frogChange()
	trainInit()
	shootInit()
	uiInit()
	uiStage()
	player.hunger = bx/2
end

function toggleVibrate()
	if Vibrate then Vibrate = false else Vibrate = true end
end

function pause()
	if not Paused then Paused = true; State = "pause" else Paused = false; State = "play" end
end

function newGame()
	reset()
	State = "play"
	Paused = false
end

function gameOver()
	State = "gameover"
	player.done = true
	spawning = false
	player.img = ded
	if Vibrate then love.system.vibrate(0.1) end
	Gameover:play()
end

function gameWin()
	Level = Level + Difficulty
	gStage = gStage + 1
	uiStage()
	player.hunger = bx/2
	if gStage == winStage then
		player.done = true
		spawning = false
		State = "win"
		Levelclear:play()
	else
		Stageclear:play()
	end
end

--####--





